% No-arbitrage model of term structure with 3 latent factor. 
% By maximum likelihood estimation (Chen-Scott (1993) method).
% A0(3) with constant or time-varying risk price.
% 
% Variables
% m03-y5: 6 yields,  DATA: FamaBliss Sample: 1958Q2-2012Q4.
%
% The structure of the file is the following:
% I. ESTIMATION 
% A. Data preparation and model selection
% B. Estimation
%    In each period:
%    1. Define data
%    2. Define initial value of parameters used for MLE
%    3. MLE, call likelihood function llfn_A03.m
%    4. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate as initial values for maincode
%
% Model specification:
% 1) Parameters.
%    Three latent mean-zero state variables. VAR(1) coefficient "phi" lower 
%    triangular; variance covariance matrix "sigma/sig" of VAR innovation
%    diagonal and normalised to eye(K)*10^(-6); Constant of short rate equation "del0" is set to the mean 
%    of short rate.
% 2) Measurement error.
%    According to Chen-Scott (1993) MLE method, three (same as # of latent
%    variables) yields are assumed to be measured without error. We propose
%    here different specifications of this triple (1 short, 1 medium, 1
%    long) of yields in the selection vector "ne" and "neorder". We show 
%    here the estimation of "ne1" with "neorder"(3month 2year 10year).

clear all; close all; clc
addpath('.\data;','.\Library;','.\Results;')
%%           *****************************************
%               *       A. DATA PREPARATION  &  
%               *       MODEL SPECIFICATION  
%               ****************************************

% Define path and load data
date          = xlsread('MainDataset.xls','Quarterly','A358:A577');    % 1960Q1 - 2014Q4
TB3MS         = xlsread('MainDataset.xls','Quarterly','AY358:AY577');  % 3-month Tbill annulized yield, from Fred St. Louis   
ydata_FB      = xlsread('MainDataset.xls','Quarterly','AF358:AJ573');  % Fama-Bliss zero Coupon Bonds 1957q2-2012q4 %(CRSP)
ydata_GSW     = xlsread('MainDataset.xls','Quarterly','AY358:BN577');  % Gurkaynak, Sack and Wright(2006), Maturity 1-15year
% my            = xlsread('MainDataset.xls','Quarterly','Z358:Z600');
load LNfactor_Q.mat               % save from the code LNfactor.m
LNreal        = -LNfactor_Q(:,2); % real activity factor 
LNinf         = -LNfactor_Q(:,3); % pricing factor 

beg_smpl      = find(date==19613);% Fama-Bliss: 1-5 maturity, 1960Q1
                                  % GSW: 1-7 year maturity, 1961Q3
                                  % GSW: 8-10 year maturity, 1971Q3
                                  % GSW: 11-15 year maturity,1971Q4
end_smpl      = find(date==20134);% 20134

% Yields
y3m           = ydata_GSW(beg_smpl:end_smpl,1)/400;            % per quarter
yields_GSW    = ydata_GSW(beg_smpl:end_smpl,2:end)./400;           % per quarter
% Annualized in percentage
numMat        = [1 2 3 4 5];      % number of maturities. It can be 1:1:15
yields        = [y3m yields_GSW(:,numMat)].*400;
GSWdata       = yields;
Time_GSW      = 1961+3/4 : 1/4 : 1981+2/4; %mroll=80 (20 years)

% Set global some parameters and intermediate result for global using or
%   the purpose of checking.
global yy yn maturity K K2;  global ne er sig;  global mu lam1 trp;

% Define yield structure used in the estimation
yname =  ['m03';'y01';'y02';'y03';'y04';'y05'];
maturity = [ 1;    4;    8;    12;    16;   20];

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
yn     = size(yname,1);    % number of yields
whole  = 1;                %Estimation for the whole sample
LENGTH = round((1981+2/4 - (1961+3/4) +1/4)*4); 
mnlags = 1;                % default 1. %maximal number of lags
trp    = 1;                % 0: constant risk price; %
                           % 1: diagonal time varying risk price
                           % 2: full matrix time varying risk price
START  = round((1961+3/4 - (1961+3/4) +1/4)*4); 
K      = 3;                % number of factors 
K2     = 3;                % number of mean-zero factor(s)
mu     = zeros(K2,1);      % preset mean zero intercept of state VAR.
lam1   = zeros(K,K);       % preset time-varying risk prices to be zeros.
sig    = eye(K)*10^(-6);   % normalise var-cov of VAR
tX     = 8;                % tolerance level of parameters precision from estimation

neorder= 1;                % select the specification of yields measured w/o error
ne     = [1 3 6];          %yields measured without error: m03 y02 y5
er     = [2 4 5];

yield= GSWdata(START:LENGTH, :);  % Select yields from the data sheet.

%%            ***********************************
%                         B.  ESTIMATION         
%             ***********************************
% ooooooooooooooooo MAIN LOOP: estimation ooooooooooooo

%:::: 1e/f. DEFINE DATA to be used for yields and states
yy = yield'./400; % transform to annualised return per dollar.
nlags = mnlags;

%:::: 2e. Set starting value for MLE.
if trp == 0;
    % phi lower triangular
    phi_11 = 0.99;
    phi_21 = 0;    phi_22 = 0.9;
    phi_31 = 0;    phi_32 = 0;     phi_33 = 0.7;
    % del1: factor loadings on short rate
    del1_1 = 1;    del1_2 = 1;     del1_3 = 1;
    % constant risk price, start from risk neutral case
    lam0_1 = 0;
    lam0_2 = 0;
    lam0_3 = 0;
    
    % Vectorise all parameter for MLE input
    bigthet = [  phi_11...
        phi_21 phi_22...
        phi_31 phi_32 phi_33...
        del1_1 del1_2 del1_3...
        lam0_1 lam0_2 lam0_3]';
    
elseif trp == 1;
    fname = [cd,'\Results\InitialValue\A03_trp',num2str(trp-1),'_ne',num2str(neorder)];
    load(fname);
    bigthet = [thetstar; zeros(K,1)];
    clear thetstar fstar;
elseif trp == 2;
    fname = [cd,'\Results\InitialValue\A03_trp',num2str(trp-1),'_ne',num2str(neorder)];
    load(fname);
    bigthet = [thetstar; reshape(diag(thetstar(13:15,1)),K^2,1)];
    clear thetstar fstar;
end;

%:::: 3e. MLE (maximize likelihood estimation)
warning off all;
% set options.
options = optimset('Display','iter','MaxIter',(0.3+trp*0.1)*(1e+005),'MaxFunEvals',1e+5,'TolFun',10^(-tX+1),'TolX',10^(-tX) );
% likelihood maximisation, call likelihood function.
% estimated parameter values:
thetstar = fminsearch('llfn_A03',bigthet,options);
% correponding likelihood:
fstar = llfn_A03(thetstar)

%% save results
fname = [cd,'\Results\InitialValue\A03_trp',num2str(trp),'_ne',num2str(neorder)];
save(fname, 'thetstar','fstar')
