%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Program of quarterly estimation for the Paper "Demography and the Behaviour of Interest Rates" (2010) by Carlo A. Favero, 
%   Arie E. Gozluklu and Haoxi Yang, Bocconi University
%   This is based on the program for paper "Term Structure Forecast: No-arbitrage Restrictions vs.
%   Large Information set" (2010) by Carlo A. Favero, Linlin Niu and Luca 
%   Sala, IGIER, Bocconi University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Maximum likelihood estimation: likelihood function for three latent yield
% factors with one or two observable factores with constant or time-varying
% risk price.
%
% "llfn": mixed factor, lower triangular fai with yield factors
% ordered first. 2 observed; 3 unobserved.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on 
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Divide the measurement equation into two groups: ne, er.
% 5. Extract latent variables from the measurement equation of ne yields.
% 6. Define matices for the state dynamics.
% 7. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.

function llfn = llfn(bigthet)

% define variables and parameters

  global yy Xo Xu Xfor maturity meanX K K2 my_r my;
  global AA1 BB1 ne er StateMeanZero demog;
  global mu phi lam1 trp DELMY unilev;
  
  if unilev == 1
      bigthet(13) = 1;
      bigthet(14) = 0; bigthet(15) = 0; %bigthet(3) = 0; bigthet(8) = 0;
      bigthet(18) = 0; bigthet(23) = 0; %bigthet(11) = 0; bigthet(12) = 0;
  end
%   demog = 1;
  
  bigt = length(yy);

%% 1. Receive initial value of parameters.

phi = reshape(bigthet(1:K^2),K,K); 

if StateMeanZero == 1;
    mu = zeros(K,1);
else
    mu = (eye(K)-phi)*meanX;
end;

sig = eye(K)*10^(-6); 
if K-K2==1;
    sig(1,1)= bigthet(K^2+1);
elseif K-K2==2;   
    sig(1:K-K2,1:K-K2) = [bigthet(K^2+1) bigthet(K^2+2)
                      bigthet(K^2+2) bigthet(K^2+K2)]; 
end

del1 = bigthet(K^2+0.5*(K-K2)*(K-K2+1)+1:K^2+0.5*(K-K2)*(K-K2+1)+K);

if demog == 0
    del0 = mean(yy(1,:))-mean(Xo,2)'*del1(1:(K-K2));
else
    DELMY=[bigthet(end);bigthet(end);bigthet(end);bigthet(end);bigthet(end);bigthet(end)];
    del0 = mean(yy(1,:))-mean(Xo,2)'*del1(1:(K-K2))-mean(-my_r(:,1))*DELMY(1);
end
lam0 = bigthet(K^2+0.5*(K-K2)*(K-K2+1)+K+1 : K^2+0.5*(K-K2)*(K-K2+1)+2*K);

if trp == 1;
    lam1 = diag(bigthet(K^2+0.5*(K-K2)*(K-K2+1)+2*K+1: K^2+0.5*(K-K2)*(K-K2+1)+3*K));
elseif trp == 2;
    lam1 = reshape(bigthet(K^2+0.5*(K-K2)*(K-K2+1)+2*K+1:K^2+0.5*(K-K2)*(K-K2+1)+2*K+K^2),K,K);
end;
 

%% 2. Calculate the PDE of coefficients for yields with all maturity on 
%     the supposed measurement equations;

% Factor loading matrix AA, BB for yields up to 120 periods
AA0 = zeros(1, 20);  
BB0 = zeros(K, 20);  
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 20;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - sig*lam0) + 0.5*BB0(:,i-1)'*sig*BB0(:,i-1) - del0;
    BB0(:,i) = (phi' - lam1'*sig')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;


%% 3. Define measurement equation for available yields in our data set.

AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';


%% 4. Divide the measurement equation into two groups: ne, er.
%     Devide the yields into yne(measured without error) and 
%     yer(measured with error) and reshape the AA, BB coefficients
%     for these yields respectively.
yne = yy(ne, :);
yer = yy(er, :);  

AAne = AA1(ne,:);
BBne = BB1(ne,:);

BBne_o=BBne(:, 1:(K-K2));
BBne_u=BBne(:, (K-K2+1):K);
ABne_o = [AAne BBne_o];  % write the coefficient in one vector
  
AAer = AA1(er, :);
BBer = BB1(er, :); 
ABer = [AAer BBer];
  

%% 5. Extract latent variables from the measurement equation of ne yields.
XXo = [ones(1, bigt); Xo];   % add one as the first variable
if demog == 0
    Xu = BBne_u\eye(size(BBne_u,1))*(yne - ABne_o * XXo);
else
    Xu = BBne_u\eye(size(BBne_u,1))*(yne - ABne_o * XXo-diag([DELMY(1);DELMY(3);DELMY(6)])*[-my_r';-my_r';-my_r']);
end

%% 6. Define matices for the state dynamics.
X = [Xo; Xu]; % LHS
XX = [XXo; Xu]; % RHS
Xfor = XX(:,bigt);  %the last period state variable for forecast
%Xu = Xu(:, H+1:bigt); % Xu corresponding to yy(:, H+1:bigt);

%% 7. Calculate log likelihood function of our system.
% Term one: -(T-1)log|det(J)|, where det(J) = det(BBne)
  t1 = -(bigt - 1) * log(abs(det(BBne_u)));
  
% Term two: -(T-1)/2 * log(det(sig))
  t2 = -.5*(bigt-1)*log(det(sig));
  
% Term three: -.5* sum[(X_t - mu - phi*X_t-h)'*inv(sig)*(X_t - mu -
% phi*X_t-h)]
  muphi = [mu phi];
  ita = X(:, 2:bigt) - muphi * XX(:,1:(bigt-1));
  sqrX = ita'*inv(sig)*ita;  % element of factor error decomposition 
  t3 = -.5 * sum(diag(sqrX));
  
% Term four: -.5*(T-1)*log(sum(i=1:N-K2, sigma(i)^2);
% sigma(i)^2 is the variance of yield (er) prediction error 
if demog == 0
    um = yer(:, 2:bigt) - ABer * XX(:, 2:bigt);
else
    um = yer(:, 2:bigt) - ABer * XX(:, 2:bigt)-diag([DELMY(2);DELMY(4);DELMY(5)])*[-my_r(2:bigt)';-my_r(2:bigt)';-my_r(2:bigt)'];
end
  VC = (1/(bigt-1))*um*um'; % variance-covariance matrix of yer
  V = diag(diag(VC));
  t4 = -.5*(bigt-1)*log(det(V));
  
% Term five: -.5*(sum t=2:T (sum i=1:yn-K2 um(t,i)^2/sigma(i)^2))
  % um = um(:,H+1:bigt);
  umsq = um.^2;
  umsqw = V\umsq;  % weighted square residual;
  sum_umsqw = sum(sum(umsqw,2));
  t5 = -.5 * sum_umsqw;
  
  llfn = -(t1 + t2 + t3 + t4 + t5);
  

  % penalize constraint violations 
  if sum(del1(K-K2+1:K)<0)>0; %to have del1 loadings on latent variables non-negative.
      llfn = real(llfn) + 1e8;
  end
  if sum(diag(sig)<0)>0;
    llfn = real(llfn) + 1e8;
  end
  if abs(imag(llfn)) > 0
    llfn = real(llfn) + 1e8;
  end
  if sum(diag(schur(phi))>1)>0
      llfn = real(llfn) + 1e8;
  end
