%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Paper Replication for Affine Model Estimation
%   "Demography and the Behaviour of Interest Rates" (2011) by  
%   Carlo A. Favero, Arie E. Gozluklu and Haoxi Yang, Bocconi University
%---------------------------------------------------------------------------------------------------------------------------------------------------------
% No-arbitrage model of term structure with 3 latent factor and two observable macro variables 
% (output/inflation factors (Ludvigson&Ng, 2009)). and middle/young ratio (Favero, Gozluklu and Tamoni, 2010)
% By maximum likelihood estimation (Chen-Scott (1993) method).
% Computes Robust Variance-Covariance Matrix for MLE Estimators
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file estimate a mixed factor model with:
% - constant risk price, set trp = 0.
% - time-varying risk price, if set trp = 1 (diagonal lambda1) or trp = 2
% (full lambda1).
%
% Rolling estimation out-of sample, 120 periods, from 1983:1 to 2014:4.
% Estimation window length 100, 1958:1-1982:4
% Iterated forecast. 
%
% Variables
% q1-y5: 6 yields
% DATA: - MainDataset.xls,'Quarterly', range: 1957:1-2008:4.
%     
% The structure of the file is the following:
% I. ESTIMATION (forecast set to 0)
% A. Data preparation and model selection
% B. Estimation (100 periods rooling window estimation)
%     In each period:
%     1. Define data
%     2. Define initial value of parameters used for MLE
%     3. MLE, call likelihood function llfn.m
%       for robust variance covariance estimation of parameters call
%       llfn_sd.m
%     4. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
%
% or
%
% II.FORECAST (forecast set to 1)
% A. Data preparation and model selection
% B. Forecast (60 periods estimation)
%    In each period:
%    1. Define data
%    2. Use estimated parameters in the likelihood function to price bonds,
%    extract latend factors, predict future yields.
% C. Save forecast for later analysis.

%
% Model specification:
% 1) Parameters.
%    Two macro variables and three latent mean-zero yield factors. VAR(1) coefficient "phi" lower 
%    block lower triangular; variance covariance matrix "sigma/sig" of VAR
%    innovation for observables is not restricted, but that for latent var.
%    equations is diagonal and normalised to be
%    eye(K2)*10^(-6); Constant of short rate equation "del0" is set to the mean 
%    of short rate minus the contributions from observable factors;
%    according to Pericoli&Taboga(2008), del1 set to positive for
%    identification.
% 2) Measurement error.
%    According to Chen-Scott (1993) MLE method, three (same as # of latent
%    variables) yields are assumed to be measured without error. We propose
%    here different specifications of this triple (1 short, 1 medium, 1
%    long) of yields in the selection vector "ne" and "neorder". We show 
%    here the estimation of "ne1" with "neorder"(1quarter 2year 5year).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc; warning off
addpath('.\data;','.\Library;','.\Results;','.\Results\Estimation;')
    
% Set global some parameters and intermediate result for global using or
% the purpose of checking.
global yy Xfor yn maturity meanX K K2 my_r Xstate Xo my Xu; 
global AA1 BB1 ne er StateMeanZero k Roll demog sig;
global mu phi lam1 trp yield DELMY unilev;
forecast = 0;  % 0: estimation
               % 1: forecast
wholesmpl= 0;  % 0: subsample estimation, window size 120
               % 1: whole sample estimation
trp      = 1;  % 0: constant risk price; 
               % 1: diagonal time varying risk price
               % 2: full matrix of time varying risk price               
demog    = 0;  % 0: without demographic variable MY
               % 1: with demographic varible MY
unilev   = 1;  % 0: unrestricted cas
               % 1: restricte the first unobserved factor (level) follows unit
               % root process
rest=1;        % Parameters to initialize, rest=1: use only delta2
% macro=0; % only for Adrian&Moench code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%            *****************************************
%                A. Data and Model Specification  
%             *****************************************

% Define path and load data
date          = xlsread('MainDataset.xls','Quarterly','A358:A577');    % 1960Q1 - 2013Q4
TB3MS         = xlsread('MainDataset.xls','Quarterly','AY358:AY577');  % 3-month Tbill annulized yield, from Fred St. Louis   
ydata_FB      = xlsread('MainDataset.xls','Quarterly','AF358:AJ573');  % Fama-Bliss zero Coupon Bonds 1957q2-2012q4 %(CRSP)
ydata_GSW     = xlsread('MainDataset.xls','Quarterly','AY358:BN577');  % Gurkaynak, Sack and Wright(2006), Maturity 1-15year
my            = xlsread('MainDataset.xls','Quarterly','Z358:Z600');
GDP_DEF       = xlsread('MainDataset.xls','Quarterly','AD306:AD577');  % GDP Deflator, 1947Q1-2013Q4
Infl_ann      = [zeros(3,1);(GDP_DEF(4:end)- GDP_DEF(1:end-3))./GDP_DEF(1:end-3)]*100; % annulized inflation
date_Infl     = xlsread('MainDataset.xls','Quarterly','A306:A577');    % 1960Q1 - 2013Q4
load LNfactor_Q.mat               % save from the code LNfactor.m
LNreal        = -LNfactor_Q(:,2); % real activity factor 
LNinf         = -LNfactor_Q(:,3); % pricing factor 

% Choose estimation smaple
beg_smpl      = find(date==19613);% Fama-Bliss: 1-5 maturity, 1960Q1
                                  % GSW: 1-7 year maturity, 1961Q3
                                  % GSW: 8-10 year maturity, 1971Q3
                                  % GSW: 11-15 year maturity,1971Q4
end_smpl      = find(date==20134);% 20134

% Yields
y3m           = ydata_GSW(beg_smpl:end_smpl,1)/400;            % per quarter
yields_GSW    = ydata_GSW(beg_smpl:end_smpl,2:end)./400;       % per quarter

% Annualized in percentage
numMat        = [1 2 3 4 5];      % number of maturities. It can be 1:1:15
yields        = [y3m yields_GSW(:,numMat)].*400;
GSWdata       = yields;
% Infl_ann      = Infl_ann(beg_smpl:end_smpl,1)*100;

date          = date(beg_smpl:end_smpl);
MAdata        = [LNinf(beg_smpl:end_smpl) LNreal(beg_smpl:end_smpl)];   % 1961Q3 - 2014Q4   
my            = my(beg_smpl:end);

% y1y           = -log(ydata_FB(beg_smpl:end_smpl,1)./100)/4;    % per quarter
% y2y           = -(1/2)*log(ydata_FB(beg_smpl:end_smpl,2)./100)/4;
% y3y           = -(1/3)*log(ydata_FB(beg_smpl:end_smpl,3)./100)/4;
% y4y           = -(1/4)*log(ydata_FB(beg_smpl:end_smpl,4)./100)/4;
% y5y           = -(1/5)*log(ydata_FB(beg_smpl:end_smpl,5)./100)/4;
% FBdata        = [y3m y1y y2y y3y y4y y5y]*400;
% % corr(GSWdata(:,2),FBdata(:,2)) = 0.985; corr(GSWdata(:,6),FBdata(:,6)) = 0.993;

% figure(1)
% Time = 1961+3/4:1/4:2013+4*(1/4);
% [AX,H1,H2] = plotyy(Time, GSWdata,Time, ones(size(Time,2),1)./my(beg_smpl:end_smpl)); 
% set(get(AX(1),'Ylabel'),'String','Yields') 
% set(get(AX(2),'Ylabel'),'String','Inverted MY','Color','k') 
% set(AX(1),'YLim',[0 16])
% set(AX(2),'YLim',[0.7 2.2])
% set(H1,'LineWidth',2.5)
% set(H2,'LineWidth',3)
% set(H2,'Color','k')
% set(AX,'xlim',[1961,2014]);
% legend('3-month Tbill yield','1-year bond yield','2-year bond yield','3-year bond yield',...
%              '4-year bond yield','5-year bond yield','MY (inverted, right-scaled)')

%%
% % % fname = [cd, '\data\RepRealRate'];
% % % save(fname, 'date','date_Infl','GSWdata','Infl_ann','MAdata','my') % keep the data file for RepRealRate.m

%%  initial setting for estimation
                
% Define yield structure used in the estimation
yname         = ['m03';'y01';'y02';'y03';'y04';'y05'];
maturity      = [ 1;    4;    8;    12;    16;   20]; 

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
yn     = size(yname,1);      % number of yields
Roll   = 1;                  % rolling estimation.
LENGTH = round((2013+4/4 - (1961+3/4) +1/4)*4); 
mnlags =1;                   % maximal number of lags
START  = round((1961+3/4 - (1961+3/4) +1/4)*4); 
if wholesmpl == 1            % whole sample: 200, the estimation is start from 1964Q1
    mRoll  = 204;            % subsmaple:    80, forcasting 214-80
else
    mRoll  = 80;             % 100;  trp=0&demog=1, mRoll = 79, since 80 not converge
end                         
K  = 5;                      % number of factors 
K2 = 3;                      % number of mean-zero factor(s)
if demog == 1
    tX = 7; % tolerance level of parameters precision from estimation
else
    tX = 8;
end

mac = 1;     % 1: [inflation factor; real activity factor] 
         
         

         StateMeanZero = 0; % 1: Mean-Zero observable factors
                            % 0: Non mean-zero observable factors
                   
mat = 1:1:20;
% ammat=2:1:20; % we consider 19 maturities as in Adrian&Moench (2008)
fmat=[1;4;8;12;16;20]; % Yield Forecast for these maturities
% yield_gws=zeros(size(param,1),size(mat,2)+1); % continuously compounded
% price=zeros(size(param,1),size(mat,2)+1); % non-log
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Yields with Error Specification
    %% ne1
    neorder=1;
    ne = [1 3 6];         %yields measured without error: m03 y02 y5
    er = [2 4 5];

                   if forecast == 0;
                       %% Initial Values from 3 Latent Factor Model
                        fname = [cd,'\Results\InitialValue\A03_trp',num2str(trp),'_ne',num2str(neorder)];
                        load(fname);
                        thetA03 = thetstar; 
                        clear thetstar fstar;
                   end
   
    yield= GSWdata(START:LENGTH, :);  % Select yields from the data sheet.
    %% uncomment, if GWS database used
    %yield= yield_gws(START:LENGTH,fmat);
    
    MAdata = MAdata(START:LENGTH,:);
%%           *********************************** 
%                    B.  Estimation / Forecast        
%            ***********************************
      
    nRoll = size(yield,1) - mRoll; %

    if forecast == 1;
        HN = [1 4 8 12 16 20];%% %selected forecasting horizon; 
        NROLL = size(yield,1) - mRoll - HN + 1; %forecast periods for each horizon in HN
        if demog == 0
            if unilev == 0
                fname = [cd,'\Results\Estimation\CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
            else
                fname = [cd,'\Results\Estimation\CS3_ll_ma2_ul',num2str(mac),'_f_trp',num2str(trp)];
            end
        else
            fname = [cd,'\Results\Estimation\CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
        end
        load(fname, 'thetall', 'fstarall');
    end;
    
    % ooooooooooooooooooooo MAIN LOOP: estimation/forecast    oooooooooooooooooooo
    k=1; 
    while k <= nRoll; 
     
          %:::: 1. DEFINE DATA to be used for yields and states 
          yy = yield(1+(k-1)*Roll:mRoll+k-1, :)'./400;   % transform to annualised return per dollar.    
          infl = MAdata(1 + (k-1)*Roll:mRoll+k-1, 1)'/400;            
          if mac == 1; % output factor(LNG, 2009)
              un = MAdata(1 + (k-1)*Roll:mRoll+k-1, 2)'./400;
              my_r= my(1 + (k-1)*Roll:mRoll+k-1, 1);
              my_r=(my_r-mean(my_r)); 
              Xo = [infl; un];    
          end
          
          nlags = mnlags;
          lam1  = zeros(K,K);
          meanX = [mean(Xo,2); zeros(K2,1)];
          
          if forecast == 0; % to estimate              
          %:::: 2e. Set starting value for MLE.
               if k == 1;
                   %We use the A0(3) model estimate as initial inputs, while
                   %assuming that observables are irrelevant; this will lead to
                   %likelihood at least as good as the A0(3) model, and let the
                   %observable factors find their own way into the system.
                   %phi full matrix, but assume latent factors follow A03 as
                   %the starting point, and latent and observable do not
                   %interact at lag one.
                   
                   Xo = [infl; un];
                   phi = zeros(K,K);
                   phi(1:K-K2,1:K-K2) = inv(Xo(:,1:end-1)*Xo(:,1:end-1)')*(Xo(:,1:end-1)*Xo(:,2:end)');
                   if unilev == 1
                       phi(3,3) = 1;
                       phi(4,3) = 0; phi(4,4) = thetA03(3,k);
                       phi(5,3) = 0; phi(5,4) = thetA03(5,k); phi(5,5) = thetA03(6,k);
                   else
                       phi(3,3) = thetA03(1,k);
                       phi(4,3) = thetA03(2,k); phi(4,4) = thetA03(3,k);
                       phi(5,3) = thetA03(4,k); phi(5,4) = thetA03(5,k); phi(5,5) = thetA03(6,k);
                   end
                   %sig condition 
                   sig_oo = reshape(kron(phi(1:K-K2,1:K-K2),phi(1:K-K2,1:K-K2))*reshape(diag(diag(cov(Xo'))),4,1), 2,2); 
                   % factor loadings for observable factors 
                   del1A03 = thetA03(7:9,k);
                   SwitchSign = diag(2*(del1A03>=0)-1);
                   phi(K-K2+1:K,K-K2+1:K)=SwitchSign*phi(K-K2+1:K,K-K2+1:K)*inv(SwitchSign);
                   [ehat_T,Fhat_T,lamhat_T,ve2_T]=pc_T(standard(yy'),K2);
                   if demog == 0
                       Xstate = [ones(1,mRoll);infl; un; -Fhat_T'];
                   else
                       Xstate = [ones(1,mRoll);infl; un; -Fhat_T'; -my_r'];
                   end
                   del_r1 =inv(Xstate*Xstate')*Xstate*yy(1,:)';
                   del_r4 =inv(Xstate*Xstate')*Xstate*yy(2,:)';
                   del_r8 =inv(Xstate*Xstate')*Xstate*yy(3,:)';
                   del_r12=inv(Xstate*Xstate')*Xstate*yy(4,:)';
                   del_r16=inv(Xstate*Xstate')*Xstate*yy(5,:)';
                   del_r20=inv(Xstate*Xstate')*Xstate*yy(6,:)';
                   
                   if demog == 1
                       delmy1=del_r1(end);
                       delmy4=del_r4(end);
                       delmy8=del_r8(end);
                       delmy12=del_r12(end);
                       delmy16=del_r16(end);
                       delmy20=del_r20(end);
                       if rest==0
                           delmy=[delmy1;delmy4;delmy8;delmy12;delmy16;delmy20];
                       else
                           delmy=[delmy1;delmy1;delmy1;delmy1;delmy1;delmy1];
                       end
                   end
                   del1 = [zeros(K-K2,1); abs(del1A03)];
                   % constant risk price, start from risk neutral and A0(3)
                   lam0 = [zeros(K-K2,1); thetA03(10:12,k)];
                   
                   % Vectorise all parameter for MLE input
                   if trp == 0;
                       if demog == 0
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0];
                       else
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0;abs(delmy(1))];
                       end
                   elseif trp == 1;
                       lam1diag = [zeros(K-K2,1); thetA03(13:15,k)];
                       if demog == 0
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0;
                               lam1diag];
                       else
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0;
                               lam1diag;abs(delmy(1))];
                       end
                   elseif trp == 2;
                       lam1(3:5,3:5) = reshape(thetA03(13:21,k),3,3);
                       if demog == 0
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0;
                               reshape(lam1,K^2,1)];
                       else
                           bigthet = [ reshape(phi,K^2,1);
                               sig_oo(1,1); sig_oo(2,1); sig_oo(2,2);
                               del1;
                               lam0;
                               reshape(lam1,K^2,1);abs(delmy(1))];
                       end
                   end;
                   clear thetA03;
               else
                  % recieve previous estimate as initial value
                  if unilev == 1
                      bigthet = thetstar;
                      bigthet(13) = 1;
                      bigthet(14) = 0; bigthet(15) = 0; %bigthet(3) = 0; bigthet(8) = 0;
                      bigthet(18) = 0; bigthet(23) = 0; %bigthet(11) = 0; bigthet(12) = 0;
                  else
                      bigthet = thetstar;
                  end
               end;                  

               %:::: 3e. MLE (maximize likelihood estimation)
               warning off all;
               % set options
               if k < 3;
                   options = optimset('Display','off','MaxIter',1.6*1e+008,'MaxFunEvals',1e+9,'TolFun',10^(-tX+1),'TolX',10^(-tX)  );
               else
                   if trp == 2;
                       options = optimset('MaxIter', ceil(0.9+trp*0.4)*1e+008,'MaxFunEvals',1e+9,'TolFun',10^(-tX+1),'TolX',10^(-tX) );
                   else
                       options = optimset('MaxIter', ceil(0.9+trp*0.25)*1e+008,'MaxFunEvals',1e+9,'TolFun',10^(-tX+1),'TolX',10^(-tX) );
                   end
               end
               % likelihood maximisation, call likelihood function.
               % estimated parameter values:
                   thetstar = fminsearch('llfn',bigthet,options);
                   fstar = llfn(thetstar)
               % correponding likelihood:
               
               % collect results for each period:
               thetall(:,k)= thetstar;
               fstarall(k) = fstar;
           
               k;
          
               %:::: 4e.Control result, either repeat the estimation with 
               %        another initial value or accept the estimate and 
               %        move forward.
               if k>4
                  fdiff = fstar-fstarall(k-1);
                  if fdiff > 800;
                     k=k; 
                     thetstar = [thetall(1:15,k-1); zeros(size(thetstar,1)-15,1)];
                  else
                     k=k+1;
                  end;
               else
                  k = k+1;
               end;

               %save intermediate result to file.
               if demog == 0
                   if unilev == 0
                       fname = [cd, '\Results\Estimation\CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
                   else
                       fname = [cd, '\Results\Estimation\CS3_ll_ma2_il',num2str(mac),'_f_trp',num2str(trp)];
                   end
               else
                   fname = [cd, '\Results\Estimation\CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
               end
               save(fname, 'thetall', 'fstarall');
          
          elseif forecast == 1;

               %:::: 2f. Input the previous estimation to likelihood function in order to 
               %        calculate coefficients on measurement equation and latent
               %        factors
                   fstar = llfn(thetall(:,k));

               %:::: 3f. Iterated forecast for horizons up to 20 quarters.
               muphi = [mu phi];
               muphicon = [1 zeros(1,size(muphi,2)-1); muphi];
               % AR(1) forecasts
                Xo_inf=[ones(mRoll,1) Xo(1,:)'];  Xo_gt=[ones(mRoll,1) Xo(2,:)'];
                Xo_inf_l=trimr(lag(Xo_inf,1),0,1);
                Xo_inf=trimr(Xo_inf,1,0);
                ar_inf = ols(Xo_inf(:,2),Xo_inf_l); 
                ar_inf_m  = ar_inf.beta(1);
                ar_inf_p  = ar_inf.beta(2);
                Xo_gt_l=trimr(lag(Xo_gt,1),0,1);
                Xo_gt=trimr(Xo_gt,1,0);
                ar_gt = ols(Xo_gt(:,2),Xo_gt_l); 
                ar_gt_m  = ar_gt.beta(1);
                ar_gt_p  = ar_gt.beta(2);
                cons_inf=0; cons_gt=0; 
               for ihor = 1:20;
                   Xhatk(:,ihor)=(muphicon^ihor)*Xfor;
                   cons_inf=cons_inf+ ar_inf_p^ihor*ar_inf_m;
                   cons_gt=cons_gt+ ar_gt_p^ihor*ar_gt_m;
                   xhat_ar(1,k+ihor-1,ihor)= cons_inf +ar_inf_p^ihor*Xfor(2)*400;
                   xhat_ar(2,k+ihor-1,ihor)= cons_gt +ar_gt_p^ihor*Xfor(3)*400;
                   my_c=-my(mRoll+k+ihor,1)-mean(-my(1:mRoll+k+ihor));
                   if demog == 0
                       yhatk(:,ihor)=[AA1 BB1]*[Xhatk(:,ihor)];
                   else
                       yhatk(:,ihor)=[AA1 BB1 DELMY]*[Xhatk(:,ihor);my_c];
                   end
                   xhat(:,k+ihor-1,ihor)=Xhatk(:,ihor).*400;
                   yhat(:,k+ihor-1,ihor)=yhatk(:,ihor).*400; % transform the magnitude back to 
                                               % percentage point as in data set
               end;
               
               k 
               k = k + 1;
          end;           
          
    end; %% end "while" for k
    
    %oooooooooooooooo  END MAIN LOOP: rolling estimation/forecast  ooooooooooooooo

%%          ***********************************
%                    C. Save the Results    
%           ***********************************
if forecast == 0; 
    if demog == 0
        if unilev == 0
            fname = [cd, '\Results\Estimation\CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
        else
            fname = [cd, '\Results\Estimation\CS3_ll_ma2_ul',num2str(mac),'_f_trp',num2str(trp)];
        end
    else
        fname = [cd, '\Results\Estimation\CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
    end
    save(fname, 'thetall', 'fstarall');

elseif forecast == 1;
    if demog == 0
        if unilev ==0 
        yhs = yhat(:,:,HN);
        xhs = xhat(2:3,:,HN);
        xhars=xhat_ar(1:2,:,HN);
        ForecastName = [cd,'\Results\forecast\yh_CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastName, 'yhs');
        ForecastMName = [cd,'\Results\forecastM\xh_CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastMName, 'xhs');
        ForecastMarName = [cd,'\Results\forecastM\xhar_CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastMarName, 'xhars');
        else
        yhs_ul = yhat(:,:,HN);
        xhs_ul = xhat(2:3,:,HN);
        xhars_ul=xhat_ar(1:2,:,HN);
        ForecastName = [cd,'\Results\forecast\yh_CS3_ll_ma2_ul',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastName, 'yhs_ul');
        ForecastMName = [cd,'\Results\forecastM\xh_CS3_ll_ma2_ul',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastMName, 'xhs_ul');
        ForecastMarName = [cd,'\Results\forecastM\xhar_CS3_ll_ma2_ul',num2str(mac),'_f_trp',num2str(trp)];
        save(ForecastMarName, 'xhars_ul');
        end
    else
        yhs_t = yhat(:,:,HN);
        xhs_t = xhat(2:3,:,HN);
        xhars_t=xhat_ar(1:2,:,HN);
        ForecastName_t = [cd,'\Results\forecast\yh_CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
        save(ForecastName_t, 'yhs_t');
        ForecastMName_t = [cd,'\Results\forecastM\xh_CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
        save(ForecastMName_t, 'xhs_t');
        ForecastMarName_t = [cd,'\Results\forecastM\xhar_CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
        save(ForecastMarName_t, 'xhars_t');
    end
end

%%           ***********************************
%               D. Estimate Standard Errors
%               ***********************************
if forecast == 0

    if demog == 0
        fname = [cd, '\Results\Estimation\CS3_ll_ma2',num2str(mac),'_f_trp',num2str(trp)];
    else
        fname = [cd, '\Results\Estimation\CS3_tays1_ma2',num2str(mac),'_f_trp',num2str(trp),'_rest',num2str(rest)];
    end
    load(fname);
    theta = thetall(:,end);
    sig = eye(K)*10^(-6);
    sig(1:K-K2,1:K-K2) = [theta(K^2+1) theta(K^2+2)
                          theta(K^2+2) theta(K^2+3)];
    theta = [theta(1:K^2);theta(K^2+K-1:end)];
    t = mRoll-1;
    [H VCV std] = Robust_VCV(@llfn_sd,theta,t);
    %save intermediate result to file.
    if demog == 0
        fname = [cd, '\Results\Estimation\stderror',num2str(mac),'_f_trp',num2str(trp)];
    else
        fname = [cd, '\Results\Estimation\stderror',num2str(mac),'_my_f_trp',num2str(trp),'_rest',num2str(rest)];
    end
    save(fname, 'H', 'VCV','std');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

